/*******************************************************************************
* File Name: BlueTooth.c
* Version 2.30
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "BlueTooth.h"
#include "CyLib.h"
#if(BlueTooth_INTERNAL_CLOCK_USED)
    #include "BlueTooth_IntClock.h"
#endif /* End BlueTooth_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 BlueTooth_initVar = 0u;
#if( BlueTooth_TX_ENABLED && (BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
    volatile uint8 BlueTooth_txBuffer[BlueTooth_TXBUFFERSIZE];
    volatile uint8 BlueTooth_txBufferRead = 0u;
    uint8 BlueTooth_txBufferWrite = 0u;
#endif /* End BlueTooth_TX_ENABLED */
#if( ( BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED ) && \
     (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH) )
    volatile uint8 BlueTooth_rxBuffer[BlueTooth_RXBUFFERSIZE];
    volatile uint8 BlueTooth_rxBufferRead = 0u;
    volatile uint8 BlueTooth_rxBufferWrite = 0u;
    volatile uint8 BlueTooth_rxBufferLoopDetect = 0u;
    volatile uint8 BlueTooth_rxBufferOverflow = 0u;
    #if (BlueTooth_RXHW_ADDRESS_ENABLED)
        volatile uint8 BlueTooth_rxAddressMode = BlueTooth_RXADDRESSMODE;
        volatile uint8 BlueTooth_rxAddressDetected = 0u;
    #endif /* End EnableHWAddress */
#endif /* End BlueTooth_RX_ENABLED */


/*******************************************************************************
* Function Name: BlueTooth_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The BlueTooth_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time UART_Start() is called. This allows for
*  component initialization without re-initialization in all subsequent calls
*  to the BlueTooth_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void BlueTooth_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(BlueTooth_initVar == 0u)
    {
        BlueTooth_Init();
        BlueTooth_initVar = 1u;
    }
    BlueTooth_Enable();
}


/*******************************************************************************
* Function Name: BlueTooth_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  BlueTooth_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void BlueTooth_Init(void) 
{
    #if(BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED)

        #if(BlueTooth_RX_INTERRUPT_ENABLED && (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            (void)CyIntSetVector(BlueTooth_RX_VECT_NUM, &BlueTooth_RXISR);
            CyIntSetPriority(BlueTooth_RX_VECT_NUM, BlueTooth_RX_PRIOR_NUM);
        #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */

        #if (BlueTooth_RXHW_ADDRESS_ENABLED)
            BlueTooth_SetRxAddressMode(BlueTooth_RXAddressMode);
            BlueTooth_SetRxAddress1(BlueTooth_RXHWADDRESS1);
            BlueTooth_SetRxAddress2(BlueTooth_RXHWADDRESS2);
        #endif /* End BlueTooth_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        BlueTooth_RXBITCTR_PERIOD_REG = BlueTooth_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        BlueTooth_RXSTATUS_MASK_REG  = BlueTooth_INIT_RX_INTERRUPTS_MASK;
    #endif /* End BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED*/

    #if(BlueTooth_TX_ENABLED)
        #if(BlueTooth_TX_INTERRUPT_ENABLED && (BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            (void)CyIntSetVector(BlueTooth_TX_VECT_NUM, &BlueTooth_TXISR);
            CyIntSetPriority(BlueTooth_TX_VECT_NUM, BlueTooth_TX_PRIOR_NUM);
        #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(BlueTooth_TXCLKGEN_DP)
            BlueTooth_TXBITCLKGEN_CTR_REG = BlueTooth_BIT_CENTER;
            BlueTooth_TXBITCLKTX_COMPLETE_REG = (BlueTooth_NUMBER_OF_DATA_BITS +
                        BlueTooth_NUMBER_OF_START_BIT) * BlueTooth_OVER_SAMPLE_COUNT;
        #else
            BlueTooth_TXBITCTR_PERIOD_REG = ((BlueTooth_NUMBER_OF_DATA_BITS +
                        BlueTooth_NUMBER_OF_START_BIT) * BlueTooth_OVER_SAMPLE_8) - 1u;
        #endif /* End BlueTooth_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(BlueTooth_TX_INTERRUPT_ENABLED && (BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            BlueTooth_TXSTATUS_MASK_REG = BlueTooth_TX_STS_FIFO_EMPTY;
        #else
            BlueTooth_TXSTATUS_MASK_REG = BlueTooth_INIT_TX_INTERRUPTS_MASK;
        #endif /*End BlueTooth_TX_INTERRUPT_ENABLED*/

    #endif /* End BlueTooth_TX_ENABLED */

    #if(BlueTooth_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        BlueTooth_WriteControlRegister( \
            (BlueTooth_ReadControlRegister() & (uint8)~BlueTooth_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(BlueTooth_PARITY_TYPE << BlueTooth_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End BlueTooth_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: BlueTooth_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  BlueTooth_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void BlueTooth_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if(BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        BlueTooth_RXBITCTR_CONTROL_REG |= BlueTooth_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        BlueTooth_RXSTATUS_ACTL_REG  |= BlueTooth_INT_ENABLE;
        #if(BlueTooth_RX_INTERRUPT_ENABLED && (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            CyIntEnable(BlueTooth_RX_VECT_NUM);
            #if (BlueTooth_RXHW_ADDRESS_ENABLED)
                BlueTooth_rxAddressDetected = 0u;
            #endif /* End BlueTooth_RXHW_ADDRESS_ENABLED */
        #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */
    #endif /* End BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED*/

    #if(BlueTooth_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!BlueTooth_TXCLKGEN_DP)
            BlueTooth_TXBITCTR_CONTROL_REG |= BlueTooth_CNTR_ENABLE;
        #endif /* End BlueTooth_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        BlueTooth_TXSTATUS_ACTL_REG |= BlueTooth_INT_ENABLE;
        #if(BlueTooth_TX_INTERRUPT_ENABLED && (BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            CyIntEnable(BlueTooth_TX_VECT_NUM);
        #endif /* End BlueTooth_TX_INTERRUPT_ENABLED*/
     #endif /* End BlueTooth_TX_ENABLED */

    #if(BlueTooth_INTERNAL_CLOCK_USED)
        /* Enable the clock. */
        BlueTooth_IntClock_Start();
    #endif /* End BlueTooth_INTERNAL_CLOCK_USED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: BlueTooth_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void BlueTooth_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if(BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED)
        BlueTooth_RXBITCTR_CONTROL_REG &= (uint8)~BlueTooth_CNTR_ENABLE;
    #endif /* End BlueTooth_RX_ENABLED */

    #if(BlueTooth_TX_ENABLED)
        #if(!BlueTooth_TXCLKGEN_DP)
            BlueTooth_TXBITCTR_CONTROL_REG &= (uint8)~BlueTooth_CNTR_ENABLE;
        #endif /* End BlueTooth_TXCLKGEN_DP */
    #endif /* BlueTooth_TX_ENABLED */

    #if(BlueTooth_INTERNAL_CLOCK_USED)
        /* Disable the clock. */
        BlueTooth_IntClock_Stop();
    #endif /* End BlueTooth_INTERNAL_CLOCK_USED */

    /* Disable internal interrupt component */
    #if(BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED)
        BlueTooth_RXSTATUS_ACTL_REG  &= (uint8)~BlueTooth_INT_ENABLE;
        #if(BlueTooth_RX_INTERRUPT_ENABLED && (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            BlueTooth_DisableRxInt();
        #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */
    #endif /* End BlueTooth_RX_ENABLED */

    #if(BlueTooth_TX_ENABLED)
        BlueTooth_TXSTATUS_ACTL_REG &= (uint8)~BlueTooth_INT_ENABLE;
        #if(BlueTooth_TX_INTERRUPT_ENABLED && (BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            BlueTooth_DisableTxInt();
        #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */
    #endif /* End BlueTooth_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: BlueTooth_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 BlueTooth_ReadControlRegister(void) 
{
    #if( BlueTooth_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(BlueTooth_CONTROL_REG);
    #endif /* End BlueTooth_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: BlueTooth_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  BlueTooth_WriteControlRegister(uint8 control) 
{
    #if( BlueTooth_CONTROL_REG_REMOVED )
        if(control != 0u) { }      /* release compiler warning */
    #else
       BlueTooth_CONTROL_REG = control;
    #endif /* End BlueTooth_CONTROL_REG_REMOVED */
}


#if(BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED)

    #if(BlueTooth_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: BlueTooth_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void BlueTooth_EnableRxInt(void) 
        {
            CyIntEnable(BlueTooth_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: BlueTooth_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void BlueTooth_DisableRxInt(void) 
        {
            CyIntDisable(BlueTooth_RX_VECT_NUM);
        }

    #endif /* BlueTooth_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: BlueTooth_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void BlueTooth_SetRxInterruptMode(uint8 intSrc) 
    {
        BlueTooth_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: BlueTooth_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  BlueTooth_rxBuffer - RAM buffer pointer for save received data.
    *  BlueTooth_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  BlueTooth_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  BlueTooth_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 BlueTooth_ReadRxData(void) 
    {
        uint8 rxData;

        #if(BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_DisableRxInt();
            #endif /* BlueTooth_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = BlueTooth_rxBufferRead;
            loc_rxBufferWrite = BlueTooth_rxBufferWrite;

            if( (BlueTooth_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = BlueTooth_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;

                if(loc_rxBufferRead >= BlueTooth_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                BlueTooth_rxBufferRead = loc_rxBufferRead;

                if(BlueTooth_rxBufferLoopDetect != 0u )
                {
                    BlueTooth_rxBufferLoopDetect = 0u;
                    #if( (BlueTooth_RX_INTERRUPT_ENABLED) && (BlueTooth_FLOW_CONTROL != 0u) && \
                         (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( BlueTooth_HD_ENABLED )
                            if((BlueTooth_CONTROL_REG & BlueTooth_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only in RX
                                *  configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                BlueTooth_RXSTATUS_MASK_REG  |= BlueTooth_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            BlueTooth_RXSTATUS_MASK_REG  |= BlueTooth_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end BlueTooth_HD_ENABLED */
                    #endif /* BlueTooth_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = BlueTooth_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_EnableRxInt();
            #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */

        #else /* BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = BlueTooth_RXDATA_REG;

        #endif /* BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  BlueTooth_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn?t free space in
    *   BlueTooth_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   BlueTooth_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 BlueTooth_ReadRxStatus(void) 
    {
        uint8 status;

        status = BlueTooth_RXSTATUS_REG & BlueTooth_RX_HW_MASK;

        #if(BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH)
            if( BlueTooth_rxBufferOverflow != 0u )
            {
                status |= BlueTooth_RX_STS_SOFT_BUFF_OVER;
                BlueTooth_rxBufferOverflow = 0u;
            }
        #endif /* BlueTooth_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error
    *  condition exists, zero is returned; otherwise, character is read and
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  BlueTooth_rxBuffer - RAM buffer pointer for save received data.
    *  BlueTooth_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  BlueTooth_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  BlueTooth_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 BlueTooth_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_DisableRxInt();
            #endif /* BlueTooth_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = BlueTooth_rxBufferRead;
            loc_rxBufferWrite = BlueTooth_rxBufferWrite;

            if( (BlueTooth_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = BlueTooth_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;
                if(loc_rxBufferRead >= BlueTooth_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                BlueTooth_rxBufferRead = loc_rxBufferRead;

                if(BlueTooth_rxBufferLoopDetect > 0u )
                {
                    BlueTooth_rxBufferLoopDetect = 0u;
                    #if( (BlueTooth_RX_INTERRUPT_ENABLED) && (BlueTooth_FLOW_CONTROL != 0u) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( BlueTooth_HD_ENABLED )
                            if((BlueTooth_CONTROL_REG & BlueTooth_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only if
                                *  RX configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                BlueTooth_RXSTATUS_MASK_REG  |= BlueTooth_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            BlueTooth_RXSTATUS_MASK_REG  |= BlueTooth_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end BlueTooth_HD_ENABLED */
                    #endif /* BlueTooth_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }

            }
            else
            {   rxStatus = BlueTooth_RXSTATUS_REG;
                if((rxStatus & BlueTooth_RX_STS_FIFO_NOTEMPTY) != 0u)
                {   /* Read received data from FIFO*/
                    rxData = BlueTooth_RXDATA_REG;
                    /*Check status on error*/
                    if((rxStatus & (BlueTooth_RX_STS_BREAK | BlueTooth_RX_STS_PAR_ERROR |
                                   BlueTooth_RX_STS_STOP_ERROR | BlueTooth_RX_STS_OVERRUN)) != 0u)
                    {
                        rxData = 0u;
                    }
                }
            }

            /* Enable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_EnableRxInt();
            #endif /* BlueTooth_RX_INTERRUPT_ENABLED */

        #else /* BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

            rxStatus =BlueTooth_RXSTATUS_REG;
            if((rxStatus & BlueTooth_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO*/
                rxData = BlueTooth_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (BlueTooth_RX_STS_BREAK | BlueTooth_RX_STS_PAR_ERROR |
                               BlueTooth_RX_STS_STOP_ERROR | BlueTooth_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        #endif /* BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 BlueTooth_GetByte(void) 
    {
        return ( ((uint16)BlueTooth_ReadRxStatus() << 8u) | BlueTooth_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: BlueTooth_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left
    *  in the RX buffer
    *
    * Global Variables:
    *  BlueTooth_rxBufferWrite - used to calculate left bytes.
    *  BlueTooth_rxBufferRead - used to calculate left bytes.
    *  BlueTooth_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 BlueTooth_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_DisableRxInt();
            #endif /* BlueTooth_RX_INTERRUPT_ENABLED */

            if(BlueTooth_rxBufferRead == BlueTooth_rxBufferWrite)
            {
                if(BlueTooth_rxBufferLoopDetect > 0u)
                {
                    size = BlueTooth_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(BlueTooth_rxBufferRead < BlueTooth_rxBufferWrite)
            {
                size = (BlueTooth_rxBufferWrite - BlueTooth_rxBufferRead);
            }
            else
            {
                size = (BlueTooth_RXBUFFERSIZE - BlueTooth_rxBufferRead) + BlueTooth_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_EnableRxInt();
            #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */

        #else /* BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = ((BlueTooth_RXSTATUS_REG & BlueTooth_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

        #endif /* End BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the RX RAM buffer by setting the read and write pointers both to zero.
    *  Clears hardware RX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_rxBufferWrite - cleared to zero.
    *  BlueTooth_rxBufferRead - cleared to zero.
    *  BlueTooth_rxBufferLoopDetect - cleared to zero.
    *  BlueTooth_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *******************************************************************************/
    void BlueTooth_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* clear the HW FIFO */
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        BlueTooth_RXDATA_AUX_CTL_REG |=  BlueTooth_RX_FIFO_CLR;
        BlueTooth_RXDATA_AUX_CTL_REG &= (uint8)~BlueTooth_RX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_DisableRxInt();
            #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */

            BlueTooth_rxBufferRead = 0u;
            BlueTooth_rxBufferWrite = 0u;
            BlueTooth_rxBufferLoopDetect = 0u;
            BlueTooth_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(BlueTooth_RX_INTERRUPT_ENABLED)
                BlueTooth_EnableRxInt();
            #endif /* End BlueTooth_RX_INTERRUPT_ENABLED */
        #endif /* End BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

    }


    /*******************************************************************************
    * Function Name: BlueTooth_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  BlueTooth__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  BlueTooth__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  BlueTooth__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  BlueTooth__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  BlueTooth__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  BlueTooth_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void BlueTooth_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(BlueTooth_RXHW_ADDRESS_ENABLED)
            #if(BlueTooth_CONTROL_REG_REMOVED)
                if(addressMode != 0u) { }     /* release compiler warning */
            #else /* BlueTooth_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = BlueTooth_CONTROL_REG & (uint8)~BlueTooth_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << BlueTooth_CTRL_RXADDR_MODE0_SHIFT);
                BlueTooth_CONTROL_REG = tmpCtrl;
                #if(BlueTooth_RX_INTERRUPT_ENABLED && \
                   (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH) )
                    BlueTooth_rxAddressMode = addressMode;
                    BlueTooth_rxAddressDetected = 0u;
                #endif /* End BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH*/
            #endif /* End BlueTooth_CONTROL_REG_REMOVED */
        #else /* BlueTooth_RXHW_ADDRESS_ENABLED */
            if(addressMode != 0u) { }     /* release compiler warning */
        #endif /* End BlueTooth_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: BlueTooth_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void BlueTooth_SetRxAddress1(uint8 address) 

    {
        BlueTooth_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: BlueTooth_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void BlueTooth_SetRxAddress2(uint8 address) 
    {
        BlueTooth_RXADDRESS2_REG = address;
    }

#endif  /* BlueTooth_RX_ENABLED || BlueTooth_HD_ENABLED*/


#if( (BlueTooth_TX_ENABLED) || (BlueTooth_HD_ENABLED) )

    #if(BlueTooth_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: BlueTooth_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void BlueTooth_EnableTxInt(void) 
        {
            CyIntEnable(BlueTooth_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: BlueTooth_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void BlueTooth_DisableTxInt(void) 
        {
            CyIntDisable(BlueTooth_TX_VECT_NUM);
        }

    #endif /* BlueTooth_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: BlueTooth_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void BlueTooth_SetTxInterruptMode(uint8 intSrc) 
    {
        BlueTooth_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: BlueTooth_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the
    *  bus is available. WriteTxData sends a byte without checking for buffer room
    *  or status. It is up to the user to separately check status.
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  BlueTooth_txBuffer - RAM buffer pointer for save data for transmission
    *  BlueTooth_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  BlueTooth_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  BlueTooth_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void BlueTooth_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(BlueTooth_initVar != 0u)
        {
            #if(BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(BlueTooth_TX_INTERRUPT_ENABLED)
                    BlueTooth_DisableTxInt();
                #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

                if( (BlueTooth_txBufferRead == BlueTooth_txBufferWrite) &&
                    ((BlueTooth_TXSTATUS_REG & BlueTooth_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    BlueTooth_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(BlueTooth_txBufferWrite >= BlueTooth_TXBUFFERSIZE)
                    {
                        BlueTooth_txBufferWrite = 0u;
                    }

                    BlueTooth_txBuffer[BlueTooth_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    BlueTooth_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(BlueTooth_TX_INTERRUPT_ENABLED)
                    BlueTooth_EnableTxInt();
                #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

            #else /* BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                BlueTooth_TXDATA_REG = txDataByte;

            #endif /* End BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: BlueTooth_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to
    *  the user to handle all bits in this return value accordingly, even if the bit
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.
    *
    *******************************************************************************/
    uint8 BlueTooth_ReadTxStatus(void) 
    {
        return(BlueTooth_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_txBuffer - RAM buffer pointer for save data for transmission
    *  BlueTooth_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  BlueTooth_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  BlueTooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void BlueTooth_PutChar(uint8 txDataByte) 
    {
            #if(BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH)
                /* The temporary output pointer is used since it takes two instructions
                *  to increment with a wrap, and we can't risk doing that with the real
                *  pointer and getting an interrupt in between instructions.
                */
                uint8 loc_txBufferWrite;
                uint8 loc_txBufferRead;

                do{
                    /* Block if software buffer is full, so we don't overwrite. */
                    #if ((BlueTooth_TXBUFFERSIZE > BlueTooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Disable TX interrupt to protect variables that could change on interrupt */
                        CyIntDisable(BlueTooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    loc_txBufferWrite = BlueTooth_txBufferWrite;
                    loc_txBufferRead = BlueTooth_txBufferRead;
                    #if ((BlueTooth_TXBUFFERSIZE > BlueTooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Enable interrupt to continue transmission */
                        CyIntEnable(BlueTooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }while( (loc_txBufferWrite < loc_txBufferRead) ? (loc_txBufferWrite == (loc_txBufferRead - 1u)) :
                                        ((loc_txBufferWrite - loc_txBufferRead) ==
                                        (uint8)(BlueTooth_TXBUFFERSIZE - 1u)) );

                if( (loc_txBufferRead == loc_txBufferWrite) &&
                    ((BlueTooth_TXSTATUS_REG & BlueTooth_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    BlueTooth_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(loc_txBufferWrite >= BlueTooth_TXBUFFERSIZE)
                    {
                        loc_txBufferWrite = 0u;
                    }
                    /* Add to the software buffer. */
                    BlueTooth_txBuffer[loc_txBufferWrite] = txDataByte;
                    loc_txBufferWrite++;

                    /* Finally, update the real output pointer */
                    #if ((BlueTooth_TXBUFFERSIZE > BlueTooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntDisable(BlueTooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    BlueTooth_txBufferWrite = loc_txBufferWrite;
                    #if ((BlueTooth_TXBUFFERSIZE > BlueTooth_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntEnable(BlueTooth_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }

            #else /* BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

                while((BlueTooth_TXSTATUS_REG & BlueTooth_TX_STS_FIFO_FULL) != 0u)
                {
                    ; /* Wait for room in the FIFO. */
                }

                /* Add directly to the FIFO. */
                BlueTooth_TXDATA_REG = txDataByte;

            #endif /* End BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: BlueTooth_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: char pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole
    *  string, it will block until the entire string has been written to the
    *  transmit buffer.
    *
    *******************************************************************************/
    void BlueTooth_PutString(const char8 string[]) 
    {
        uint16 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(BlueTooth_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(string[buf_index] != (char8)0)
            {
                BlueTooth_PutChar((uint8)string[buf_index]);
                buf_index++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: BlueTooth_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void BlueTooth_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(BlueTooth_initVar != 0u)
        {
            do
            {
                BlueTooth_PutChar(string[buf_index]);
                buf_index++;
            }while(buf_index < byteCount);
        }
    }


    /*******************************************************************************
    * Function Name: BlueTooth_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void BlueTooth_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(BlueTooth_initVar != 0u)
        {
            BlueTooth_PutChar(txDataByte);
            BlueTooth_PutChar(0x0Du);
            BlueTooth_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: BlueTooth_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  BlueTooth_txBufferWrite - used to calculate left space.
    *  BlueTooth_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 BlueTooth_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(BlueTooth_TX_INTERRUPT_ENABLED)
                BlueTooth_DisableTxInt();
            #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

            if(BlueTooth_txBufferRead == BlueTooth_txBufferWrite)
            {
                size = 0u;
            }
            else if(BlueTooth_txBufferRead < BlueTooth_txBufferWrite)
            {
                size = (BlueTooth_txBufferWrite - BlueTooth_txBufferRead);
            }
            else
            {
                size = (BlueTooth_TXBUFFERSIZE - BlueTooth_txBufferRead) + BlueTooth_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(BlueTooth_TX_INTERRUPT_ENABLED)
                BlueTooth_EnableTxInt();
            #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

        #else /* BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

            size = BlueTooth_TXSTATUS_REG;

            /* Is the fifo is full. */
            if((size & BlueTooth_TX_STS_FIFO_FULL) != 0u)
            {
                size = BlueTooth_FIFO_LENGTH;
            }
            else if((size & BlueTooth_TX_STS_FIFO_EMPTY) != 0u)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: BlueTooth_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the TX RAM buffer by setting the read and write pointers both to zero.
    *  Clears the hardware TX FIFO.  Any data present in the FIFO will not be sent.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_txBufferWrite - cleared to zero.
    *  BlueTooth_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void BlueTooth_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        /* clear the HW FIFO */
        BlueTooth_TXDATA_AUX_CTL_REG |=  BlueTooth_TX_FIFO_CLR;
        BlueTooth_TXDATA_AUX_CTL_REG &= (uint8)~BlueTooth_TX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(BlueTooth_TX_INTERRUPT_ENABLED)
                BlueTooth_DisableTxInt();
            #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

            BlueTooth_txBufferRead = 0u;
            BlueTooth_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(BlueTooth_TX_INTERRUPT_ENABLED)
                BlueTooth_EnableTxInt();
            #endif /* End BlueTooth_TX_INTERRUPT_ENABLED */

        #endif /* End BlueTooth_TXBUFFERSIZE > BlueTooth_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: BlueTooth_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BlueTooth_initVar - checked to identify that the component has been
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void BlueTooth_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(BlueTooth_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(BlueTooth_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == BlueTooth_SEND_BREAK) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() |
                                                          BlueTooth_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    BlueTooth_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = BlueTooth_TXSTATUS_REG;
                    }while((tmpStat & BlueTooth_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == BlueTooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = BlueTooth_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & BlueTooth_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == BlueTooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == BlueTooth_REINIT) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT) )
                {
                    BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() &
                                                  (uint8)~BlueTooth_CTRL_HD_SEND_BREAK);
                }

            #else /* BlueTooth_HD_ENABLED Full Duplex mode */

                static uint8 tx_period;

                if( (retMode == BlueTooth_SEND_BREAK) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode*/
                    #if( (BlueTooth_PARITY_TYPE != BlueTooth__B_UART__NONE_REVB) || \
                                        (BlueTooth_PARITY_TYPE_SW != 0u) )
                        BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() |
                                                              BlueTooth_CTRL_HD_SEND_BREAK);
                    #endif /* End BlueTooth_PARITY_TYPE != BlueTooth__B_UART__NONE_REVB  */

                    #if(BlueTooth_TXCLKGEN_DP)
                        tx_period = BlueTooth_TXBITCLKTX_COMPLETE_REG;
                        BlueTooth_TXBITCLKTX_COMPLETE_REG = BlueTooth_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = BlueTooth_TXBITCTR_PERIOD_REG;
                        BlueTooth_TXBITCTR_PERIOD_REG = BlueTooth_TXBITCTR_BREAKBITS8X;
                    #endif /* End BlueTooth_TXCLKGEN_DP */

                    /* Send zeros*/
                    BlueTooth_TXDATA_REG = 0u;

                    do /* wait until transmit starts */
                    {
                        tmpStat = BlueTooth_TXSTATUS_REG;
                    }while((tmpStat & BlueTooth_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == BlueTooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = BlueTooth_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & BlueTooth_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == BlueTooth_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == BlueTooth_REINIT) ||
                    (retMode == BlueTooth_SEND_WAIT_REINIT) )
                {

                    #if(BlueTooth_TXCLKGEN_DP)
                        BlueTooth_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        BlueTooth_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End BlueTooth_TXCLKGEN_DP */

                    #if( (BlueTooth_PARITY_TYPE != BlueTooth__B_UART__NONE_REVB) || \
                         (BlueTooth_PARITY_TYPE_SW != 0u) )
                        BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() &
                                                      (uint8)~BlueTooth_CTRL_HD_SEND_BREAK);
                    #endif /* End BlueTooth_PARITY_TYPE != NONE */
                }
            #endif    /* End BlueTooth_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: BlueTooth_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void BlueTooth_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0u)
        {
            #if( BlueTooth_CONTROL_REG_REMOVED == 0u )
                BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() |
                                                      BlueTooth_CTRL_MARK);
            #endif /* End BlueTooth_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
            #if( BlueTooth_CONTROL_REG_REMOVED == 0u )
                BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() &
                                                    (uint8)~BlueTooth_CTRL_MARK);
            #endif /* End BlueTooth_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndBlueTooth_TX_ENABLED */

#if(BlueTooth_HD_ENABLED)


    /*******************************************************************************
    * Function Name: BlueTooth_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART.
    *
    * Side Effects:
    *  Disable RX interrupt mask, when software buffer has been used.
    *
    *******************************************************************************/
    void BlueTooth_LoadTxConfig(void) 
    {
        #if((BlueTooth_RX_INTERRUPT_ENABLED) && (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            /* Disable RX interrupts before set TX configuration */
            BlueTooth_SetRxInterruptMode(0u);
        #endif /* BlueTooth_RX_INTERRUPT_ENABLED */

        BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() | BlueTooth_CTRL_HD_SEND);
        BlueTooth_RXBITCTR_PERIOD_REG = BlueTooth_HD_TXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(BlueTooth_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */
    }


    /*******************************************************************************
    * Function Name: BlueTooth_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    * Side Effects:
    *  Set RX interrupt mask based on customizer settings, when software buffer
    *  has been used.
    *
    *******************************************************************************/
    void BlueTooth_LoadRxConfig(void) 
    {
        BlueTooth_WriteControlRegister(BlueTooth_ReadControlRegister() &
                                                (uint8)~BlueTooth_CTRL_HD_SEND);
        BlueTooth_RXBITCTR_PERIOD_REG = BlueTooth_HD_RXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(BlueTooth_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */

        #if((BlueTooth_RX_INTERRUPT_ENABLED) && (BlueTooth_RXBUFFERSIZE > BlueTooth_FIFO_LENGTH))
            /* Enable RX interrupt after set RX configuration */
            BlueTooth_SetRxInterruptMode(BlueTooth_INIT_RX_INTERRUPTS_MASK);
        #endif /* BlueTooth_RX_INTERRUPT_ENABLED */
    }

#endif  /* BlueTooth_HD_ENABLED */


/* [] END OF FILE */
